@@ -868,6 +868,10 @@ int CST_is_valid(CST * st, X509 * cert)
 
     if (st && cert)
     {
+        /* Treat all imported certificates as trusted */
+        if (CST_search_by_X509(st, cert) > 0)
+            return TRUE;
+
         CST_LOCK_BEGIN(LOCK_SH);
             GSList * chains = cst_get_all_chain(st, cert);
             GSList * i;
@@ -876,19 +880,25 @@ int CST_is_valid(CST * st, X509 * cert)
              * expired or not, because this is done in
              * CST_get_state */
             if (NULL == chains)
-                result = TRUE;
+                result = FALSE;
             else if (SEQNUM_FROM_POINTER(chains->data) == 0)
-                    result = TRUE;
-
-            for (i = chains; i != NULL; i = i->next)
+                /* this case means that the given X509 was a root cert, clever */
+                result = FALSE;
+            else 
             {
-                if (!result && cst_is_valid_internal(st, cert, (GSList *)(i->data)))
+                for (i = chains; i != NULL; i = i->next)
                 {
-                    result = TRUE;
+                    if (!result && cst_is_valid_internal(st, cert, (GSList *)(i->data)))
+                    {
+                        result = TRUE;
+                    }
+                    g_slist_free((GSList *)(i->data));
                 }
-                g_slist_free((GSList *)(i->data));
             }
-            g_slist_free(chains);
+
+            if (chains)
+                g_slist_free(chains);
+
         CST_LOCK_END;
     }
     else
